/****************************************************************************
*
*  $RCSfile: Clok.java,v $
*
*   Clok class.
*  Original Author: Caskey Dickson <caskey@technocage.com> 2002-04-25
*  Copyright 2002 TechnoCage, Inc.  All Rights Reserved
*  $Id: Clok.java,v 1.5 2002/05/01 07:05:32 caskey Exp $
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software Foundation, 
*   Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*****************************************************************************/


package com.technocage.clok;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import javax.swing.JFrame;
import javax.swing.JPanel;

import java.util.Date;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.Locale;
import java.util.List;
import java.util.StringTokenizer;

import java.text.SimpleDateFormat;

/**
 * A new way of looking at time.
 * Graphics elements Based on code written by Dr. Ray Toal.
 *
 * @author Caskey L. Dickson
 */
public class Clok extends JPanel {

    /**
     * Keeps track of the last point to draw the next line from. 
     * Part of the sketcher code that Clok was adapted from.
     */
    private Point lastPoint;

    /**
     * An array of strings with the time zones to be displayed.
     */
    private String[] zones;

    /**
     * Constructs a clok panel, registering listeners for the mouse.
     */
    public Clok(String[] zones) {
        this.zones = zones;
        // When the mouse button goes down, set the current point
        // to the location at which the mouse was pressed.
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                lastPoint = new Point(e.getX(), e.getY());
            }
        });

        // When the mouse is dragged, draw a line from the old point
        // to the new point and update the value of lastPoint to hold
        // the new current point.
        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                Graphics g = getGraphics();
                g.drawLine(lastPoint.x, lastPoint.y, e.getX(), e.getY());
                lastPoint = new Point(e.getX(), e.getY());
                g.dispose();
            }
        });
    }

    public class ClokEntry extends Object {
      public String zone;
      double intensities[];

      double getIntensity(int hour) {
        return intensities[hour];
      }

      void initializeIntensities(int boundaries[]) {
        intensities = new double[24];
        for(int hour = 0; hour < intensities.length; hour++) {
          int thisBoundary = 0;
          int bestDiff = 24;
          for(int j = 0; j < boundaries.length; j++) {
            if(hour >= boundaries[j]) {
              int diff = hour - boundaries[j];
              if(diff <= bestDiff) {
                thisBoundary = j;
                bestDiff = diff;
              }
            }
            if (hour < boundaries[j]) {
              int diff = hour - (boundaries[j]-24);
              if(diff <= bestDiff) {
                thisBoundary = j;
                bestDiff = diff;
              }
            }
          }
          switch(thisBoundary) {
          case(0):
              intensities[hour] = 0.00;
              break;
          case(1):
              intensities[hour] = 0.85;
              break;
          case(2):
              intensities[hour] = 1.00;
              break;
          case(3):
              intensities[hour] = 0.85;
              break;
          case(4):
              intensities[hour] = 0.50;
              break;
          default:
              intensities[hour] = 0.55;
              break;
          }
        }
      }

      int[] parseIntensities(String s) {
        int boundaries[] = new int[5];
        for(int i = 0; i < boundaries.length; i++) {
          boundaries[i] = i;
        }
        StringTokenizer st = new StringTokenizer(s, " ,");
        if(st.countTokens() > 1) {
          int counter = 0;
          while(st.hasMoreTokens()) {
            boundaries[counter++] = Integer.parseInt(st.nextToken());
          }
        } else {
          // one token, must be name
          if(s.equals("hacker")) {
            return parseIntensities(System.getProperty(hackerBoundaryProperty,
                                                       hackerBoundaries));
          } else if (s.equals("human")) {
            return parseIntensities(System.getProperty(humanBoundaryProperty,
                                                       humanBoundaries));
          } else if (s.equals("server")) {
            return parseIntensities(System.getProperty(serverBoundaryProperty,
                                                       serverBoundaries));
          } else {
            return parseIntensities(System.getProperty(boundaryPropertyRoot + s,
                                                       defaultBoundaries));
          }
        }
        return boundaries;
      }

      private static final String boundaryPropertyRoot =
        "com.technocage.clok.Clok.boundaries.";

      private static final String hackerBoundaryProperty =
        boundaryPropertyRoot + "hacker";
      private static final String hackerBoundaries =
        "4,11,16,17,22";

      private static final String serverBoundaryProperty =
        boundaryPropertyRoot + "server";
      private static final String serverBoundaries =
        "2,4,12,14,15";

      private static final String defaultBoundaryProperty =
        boundaryPropertyRoot + "default";
      private static final String defaultBoundaries =
        "2,7,12,14,18";

      private static final String humanBoundaryProperty =
        boundaryPropertyRoot + "human";
      private static final String humanBoundaries =
        "23,7,12,13,17";

      public ClokEntry(String zone) {
        this(zone, System.getProperty(defaultBoundaryProperty,
                                      defaultBoundaries));
      }

      public ClokEntry(String zone, String boundaries) {
        this.zone = zone;
        // "dark,light,lightest,light,medium"
        initializeIntensities(parseIntensities(boundaries));
      }
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        g.setColor(Color.white);

        int bX = 0;
        int bY = 0;
        int bWidth = getWidth();
        int bHeight = getHeight()/zones.length;
        bHeight--;

        for(int i = 0; i < zones.length; i++) {
          ClokEntry c;
          if(zones[i].indexOf(":") == -1) {
            c = new ClokEntry(zones[i]);
          } else {
            // okay, has boundary spec.
            int separatorPosition = zones[i].indexOf(":");
            c = new ClokEntry(zones[i].substring(0, separatorPosition),
                              zones[i].substring(separatorPosition+1));
          }
          makeClok(g, bX, bY+((bHeight+1)*i), bWidth, bHeight, c);
        }

    }

    void makeClok(Graphics g, int bX, int bY, int bWidth, int bHeight,
        ClokEntry clok) {

      for(int i = 0; i < 24; i++) {
        g.setColor(new Color((float)((0.5)*clok.getIntensity(i)), 
                             (float)((0.5)*clok.getIntensity(i)), 
                             (float)((1.0)*clok.getIntensity(i))));
        g.fillRect(bX + (i*(bWidth/24)), bY, (bWidth/24), bHeight);
        g.setColor(Color.gray.brighter());
        g.drawLine(bX + (i*(bWidth/24)), bY, bX + i*(bWidth/24), bY+bHeight);
      }

      Calendar now = Calendar.getInstance(TimeZone.getTimeZone(clok.zone));
      int hour = now.get(now.HOUR_OF_DAY);
      int min = now.get(now.MINUTE);
      // System.out.println("h:" + hour + " M:" + min);
    
      g.setColor(Color.red);
      g.drawLine(bX+(hour*(bWidth/24))+(int)(((double)bWidth/(24*60))*min), bY,
                 bX+(hour*(bWidth/24))+(int)(((double)bWidth/(24*60))*min), bY+bHeight-1);
      g.setColor(Color.gray);
      g.drawLine(bX , bY + bHeight, bX + bWidth, bY+bHeight);
    }

    /**
     * A tester method that embeds the panel in a frame so you can
     * "run" it as an application.
     */
    public static void main(String[] args) {
      if(args.length == 0) {
        System.out.println("Clok 1.1");
        String zones[] = TimeZone.getAvailableIDs();
        for(int i = 0; i < zones.length; i++) {
          System.out.println(zones[i]);
        }
      } else {
        JFrame frame = new JFrame("clok");
        frame.getContentPane().add(new Clok(args), "Center");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(24*32, 60);
        frame.show();
        while(true) {
          frame.repaint();
          try {Thread.sleep(1000*60);} catch(InterruptedException e) {}
        }
      }
    }
}
